/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* 	sddsIDL.c
*
*	Author: Bob Daly
*	Date:	April 28, 1999
*       
*       Modifications:
*/

#include <stdio.h>
#include <string.h>
#include <sddsIDL.h>

#define DEBUG 1
/* 
   Keep info about SDDS tables local to this library
   The IDL object gets a TABLE_ID index 
*/
typedef struct
{
  SDDS_TABLE table;
  char filename[255]; /*path of opened file*/
  int mode;
  int pageStarted;
  int free;
  long  currentPage;
  SDDS_ARRAY	sddsArray;
  
}
SDDS_FD;
static SDDS_FD sddsFdTable[MAX_OPEN_SDDS_FILES];
static int firstTime=0;


/*****************************************************************
 result=sddsInit
		 result == long (OK or ERROR)
 ****************************************************************/
long
  sddsInit()
 {
   /*
     Declare passed in values from IDL call
   */
   /*
     Declare local variables
   */
   int i;
   
   if(firstTime==0) {
     for(i=0;i<MAX_OPEN_SDDS_FILES;i++)
       {
	 sddsFdTable[i].free=1;
	 sddsFdTable[i].currentPage=0;
	 sddsFdTable[i].filename[0]='\0';
	 sddsFdTable[i].mode=SDDS_READ;
	 sddsFdTable[i].pageStarted=0;
	 sddsFdTable[i].sddsArray.dimension=NULL;
	 sddsFdTable[i].sddsArray.elements=0;
	 sddsFdTable[i].sddsArray.data=NULL;
	 sddsFdTable[i].sddsArray.pointer=NULL;
       }
     firstTime=1;
     
   }
   return(OK);
 }

     
 
/*****************************************************************
 result=sddsOopen,filename,mode,dataMode,status,errMsg
		 result == long (index into SDDS_FD table)
		 filename == string
		 mode     == string(READ or WRITE)
		 dataMode == string(SDDS_BINARY or SDDS_ASCII)
		 errmsg == byte array 120 bytes
 ****************************************************************/
long
  sddsOpen(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   STRING *fnStr = (STRING *)argv[0];
   STRING *modeStr = (STRING *)argv[1];
   STRING *dataModeStr =(STRING *)argv[2];
   short  *status     = (short *  )argv[3];
   char *errMsg     = (char *  )argv[4];

   /*
     Local variables
   */
   long dataMode = SDDS_ASCII;
   long mode=SDDS_READ;
   long i,fd;
   char * fileName=fnStr->s;
   char **sddsErrs;
   int32_t sddsNumErrs;

   i=sddsInit();
   
   if(!strcmp(dataModeStr->s,"BINARY")) dataMode=SDDS_BINARY;
   if(!strcmp(modeStr->s,"WRITE"))mode=SDDS_WRITE;
   fd=MAX_OPEN_SDDS_FILES+1;
   
   for(i=0;i<MAX_OPEN_SDDS_FILES;i++)
     {
       if(sddsFdTable[i].free == 1) 
	 {
	   fd=i;
	   break;
	 }
     }
   
   if(fd>MAX_OPEN_SDDS_FILES)
     {
       strcpy(errMsg,"Exceeded maximum supported SDDS files");
       *status=ERROR;
       if(DEBUG) printf("MAX_OPEN_SDDS_FILES  error<%s>\n",errMsg);
       return(-1l);
     }
   
   sddsFdTable[fd].mode = mode;
   if(mode==SDDS_READ)
     {
       if (!SDDS_InitializeInput(&(sddsFdTable[fd].table), fileName))
	 {
	   
	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strcpy(errMsg, sddsErrs[0]);
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   *status=ERROR;
	   if(DEBUG) printf("InitializeInput error<%s>\n",errMsg);
	   return(-1l);
	 }
       
       strcpy(sddsFdTable[fd].filename,fileName);
       sddsFdTable[fd].free = 0;
       
     }
   else
     {
       
       if (!SDDS_InitializeOutput(&(sddsFdTable[fd].table),
				  dataMode, 1, NULL, NULL, fileName))
	 {
	   
	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strcpy(errMsg, sddsErrs[0]);
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   *status=ERROR;
	   if(DEBUG) printf("InitializeOutput error<%s>\n",errMsg);
	   return(-1l);
	   
	 }
       
       strcpy(sddsFdTable[fd].filename,fileName);
       sddsFdTable[fd].free = 0;
       
     }
   
   errMsg[0]=(char)0;
   *status=OK;
   return(fd);
 }
		   




/*****************************************************************
    result=getNames,fd,nnames,class,status,errmsg
		 result == string (array elements separated by '*')
		 fd     == long   (index to SDDS_FD table)
		 nnames == long   (number or names returned)
		 class  == string (ARRAY, PARAMETER or COLUMN)
		 status == long   (ERROR or OK)
		 errmsg == string
 ****************************************************************/
char *
  getNames(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   long *nnames		=(long *)argv[1];
   STRING *class	=(STRING *)argv[2];
   short *status	=(short *)argv[3];
   char *errMsg		=(char *)argv[4];
   
   /*
     Local variables
   */
   char **nameData;
   char **sddsErrs;
   int32_t sddsNumErrs,number;
   int i,retlen;
   char *retStr;
   

   if(!strcmp(class->s,"column"))
     {
       if ((nameData=SDDS_GetColumnNames(&(sddsFdTable[*fd].table),&number)) 
	== NULL)
	 {
	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getColumnNames: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
         if(number > 0)
	  {
	    retlen=0;
	    for(i=0;i<number;i++)
	      {
		retlen+=strlen(nameData[i]);
		retlen++;	
	      }
	    if( (retStr=(char*)malloc(retlen))==(char*)NULL)
	      {
		strncpy(errMsg,"getColumnNames:mallc error",118);
		errMsg[119]=0;
		*status=ERROR;
		if(DEBUG) printf("getColumnNames: <%s>\n",errMsg);
		return((char*)NULL);
	      }
	    retStr[0]=0;
	    for(i=0;i<number-1;i++) {
	      strcat(retStr,nameData[i]);
	      strcat(retStr,"*");
	    }
	    strcat(retStr,nameData[number-1]);
	    for(i=0;i<number;i++) free(nameData[i]);
	  }else{
	    retStr=(char*)malloc(1);
	    retStr[0]=0;
	  }
       free(nameData);
       *nnames=number;
       *status=OK;
      return(retStr);
     }
   if(!strcmp(class->s,"parameter"))
     {
       if ((nameData=SDDS_GetParameterNames(&(sddsFdTable[*fd].table),&number)) 
	== NULL)
	 {
	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterNames: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
        if(number > 0)
	  {
	    retlen=0;
	    for(i=0;i<number;i++)
	      {
		retlen+=strlen(nameData[i]);
		retlen++;
	      }
	    if( (retStr=(char*)malloc(retlen))==(char*)NULL)
	      {
		strncpy(errMsg,"getparameterNames:mallc error",118);
		errMsg[119]=0;
		*status=ERROR;
		if(DEBUG) printf("getParameterNames: <%s>\n",errMsg);
		return((char*)NULL);
	      }
	    retStr[0]=0;
  	    for(i=0;i<number-1;i++) {
	      strcat(retStr,nameData[i]);
	      strcat(retStr,"*");
	    }
	    strcat(retStr,nameData[number-1]);
	    for(i=0;i<number;i++) free(nameData[i]);
	  }else{
	    retStr=(char*)malloc(1);
	    retStr[0]=0;
	  }
	free(nameData);
	*nnames=number;
	*status=OK;
	return(retStr);
     }
   if(!strcmp(class->s,"array"))
     {
       if ((nameData=SDDS_GetArrayNames(&(sddsFdTable[*fd].table),&number)) 
	== NULL)
	 {
	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getArrayNames: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
        if(number > 0)
	  {
	    retlen=0;
	    for(i=0;i<number;i++)
	      {
		retlen+=strlen(nameData[i]);
		retlen++;	
	      }
	    if( (retStr=(char*)malloc(retlen))==(char*)NULL)
	      {
		strncpy(errMsg,"getNames:mallc error",118);
		errMsg[119]=0;
		*status=ERROR;
		if(DEBUG) printf("getArrayNames: <%s>\n",errMsg);
		return((char*)NULL);
	      }
	    retStr[0]=0;
	    for(i=0;i<number-1;i++) {
	      strcat(retStr,nameData[i]);
	      strcat(retStr,"*");
	    }
	    strcat(retStr,nameData[number-1]);
	    for(i=0;i<number;i++) free(nameData[i]);
	  }else{
	    retStr=(char*)malloc(1);
	    retStr[0]=0;
	  }
       free(nameData);
       *nnames=number;
       *status=OK;
      return(retStr);
     }
  return((char*)NULL);
}
 
       
	   
  
    /*****************************************************************
    result=setArrayValueX,fd,name,value,sizes,errmsg
		 result == long OK or ERROR
		 fd == long (index into table)
		 name == string   (parametername)
		 value == (Double,Float,Long,Short,String,Byte)array
		 sizes == long array with dimensions elements
		 errmsg == bytarr(120)
 ****************************************************************/
long
  setArrayValueD(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   double *value	=(double *)argv[2];
   int32_t *sizes	=(int32_t *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =SDDS_CONTIGUOUS_DATA;
   
  if(!SDDS_SetArray(&(sddsFdTable[*fd].table),name->s,setMode,value,sizes))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setArrayValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
 }

long
  setArrayValueF(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   float  *value	=(float *)argv[2];
   int32_t *sizes	=(int32_t *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =SDDS_CONTIGUOUS_DATA;
   
  if(!SDDS_SetArray(&(sddsFdTable[*fd].table),name->s,setMode,value,sizes))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setArrayValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
 }

long
  setArrayValueL(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long   *value	=(long *)argv[2];
   int32_t *sizes	=(int32_t *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =SDDS_CONTIGUOUS_DATA;
   
  if(!SDDS_SetArray(&(sddsFdTable[*fd].table),name->s,setMode,value,sizes))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setArrayValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
 }

long
  setArrayValueS(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   short   *value	=(short *)argv[2];
   int32_t *sizes	=(int32_t *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =SDDS_CONTIGUOUS_DATA;
   
  if(!SDDS_SetArray(&(sddsFdTable[*fd].table),name->s,setMode,value,sizes))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setArrayValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
 }

long
  setArrayValueB(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   char   *value	=(char *)argv[2];
   int32_t *sizes	=(int32_t *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =SDDS_CONTIGUOUS_DATA;
   
  if(!SDDS_SetArray(&(sddsFdTable[*fd].table),name->s,setMode,value,sizes))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setArrayValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
 }
long
  setArrayValueString(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   STRING   *value	=(STRING *)argv[2];
   int32_t *sizes	=(int32_t *)argv[3];
   long *elements	=(long *)argv[4];
  char *errMsg		=(char *)argv[5];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long i;
   long setMode =SDDS_CONTIGUOUS_DATA;
   char	**stringDataArray;
   stringDataArray=(char **)malloc(sizeof(char *)*(*elements));
   for(i=0;i<*elements;i++)
     {
       stringDataArray[i]=value[i].s;
     }
   
  if(!SDDS_SetArray(&(sddsFdTable[*fd].table),name->s,setMode,stringDataArray,sizes))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setArrayValueD:error  <%s>\n",errMsg);
	   free(stringDataArray);
	   return(ERROR);
     }
  free(stringDataArray);
  return(OK);
 }

    /*****************************************************************
    result=setColumnValueX,fd,name,value,status,errmsg
		 result == long OK or ERROR
		 fd == long (index into table)
		 name == string   (parametername)
		 value == (Double,Float,Long,Short,String,Byte)
		 elements==long
		 errmsg == bytarr(120)
 ****************************************************************/
long
  setColumnValueD(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   double *value	=(double *)argv[2];
   long *elements	=(long *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
   
   
  if(!SDDS_SetColumn(&(sddsFdTable[*fd].table),setMode,value,*elements,name->s))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setColumnValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setColumnValueF(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   float *value		=(float *)argv[2];
   long *elements	=(long *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
 
  if(!SDDS_SetColumn(&(sddsFdTable[*fd].table),setMode,value,*elements,name->s))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setColumnValueF:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setColumnValueL(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long *value		=(long *)argv[2];
   long *elements	=(long *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
 
  if(!SDDS_SetColumn(&(sddsFdTable[*fd].table),setMode,value,*elements,name->s))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setColumnValueL:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setColumnValueS(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   short *value		=(short *)argv[2];
   long *elements	=(long *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
 
  if(!SDDS_SetColumn(&(sddsFdTable[*fd].table),setMode,value,*elements,name->s))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setColumnValueS:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setColumnValueB(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   char *value		=(char *)argv[2];
   long *elements	=(long *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
 
  if(!SDDS_SetColumn(&(sddsFdTable[*fd].table),setMode,value,*elements,name->s))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setColumnValueS:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setColumnValueString(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   STRING *value	=(STRING *)argv[2];
   long *elements	=(long *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
   char **v;
   long i;
   
   if( (v= (char **)malloc(sizeof(char*)*(*elements)))==(char **)NULL)
     {
       strcpy(errMsg,"Memory allocation error");
       if(DEBUG) printf("setColumnValueString:error  <%s>\n",errMsg);
       return(ERROR);
     }
   for(i=0;i<*elements;i++,value++)
     {
       v[i]= value->s;
     }

   if(!SDDS_SetColumn(&(sddsFdTable[*fd].table),setMode,v,*elements,name->s))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setColumnValueS:error  <%s>\n",errMsg);
	   free(v);
	   return(ERROR);
     }
  free(v);
  return(OK);
}
    /*****************************************************************
    result=setRowValueX,fd,name,value,row,errmsg
		 result == long OK or ERROR
		 fd == long (index into table)
		 name == string   (parametername)
		 value == (Double,Float,Long,Short,String,Byte)
		 row  == long
		 errmsg == bytarr(120)
 ****************************************************************/
long
  setRowValueD(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   double *value	=(double *)argv[2];
   long *row		=(long *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
    
  if(!SDDS_SetRowValues(&(sddsFdTable[*fd].table),setMode,*row,name->s,*value,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setRowValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setRowValueF(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   float *value		=(float *)argv[2];
   long *row		=(long *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
    
  if(!SDDS_SetRowValues(&(sddsFdTable[*fd].table),setMode,*row,name->s,*value,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setRowValueF:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setRowValueL(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long *value		=(long *)argv[2];
   long *row		=(long *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
    
  if(!SDDS_SetRowValues(&(sddsFdTable[*fd].table),setMode,*row,name->s,*value,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setRowValueL:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setRowValueS(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   short *value		=(short *)argv[2];
   long *row		=(long *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
    
  if(!SDDS_SetRowValues(&(sddsFdTable[*fd].table),setMode,*row,name->s,*value,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setRowValueS:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setRowValueB(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   char *value		=(char *)argv[2];
   long *row		=(long *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
    
  if(!SDDS_SetRowValues(&(sddsFdTable[*fd].table),setMode,*row,name->s,*value,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setRowValueB:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
 long
  setRowValueString(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   STRING *value	=(STRING *)argv[2];
   long *row		=(long *)argv[3];
  char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
    
  if(!SDDS_SetRowValues(&(sddsFdTable[*fd].table),setMode,*row,name->s,value->s,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setRowValueString:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
   /*****************************************************************
    result=setParameterValueX,fd,name,value,status,errmsg
		 result == long OK or ERROR
		 fd == long (index into table)
		 name == string   (parametername)
		 value == (Double,Float,Long,Short,String,Byte)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  setParameterValueD(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   double *value	=(double *)argv[2];
   char *errMsg		=(char *)argv[3];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   double v = *value;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
   
  
 
  if(!SDDS_SetParameters(&(sddsFdTable[*fd].table),setMode,name->s,v,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setParameterValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setParameterValueF(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   float  *value	=(float *)argv[2];
   char *errMsg		=(char *)argv[3];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   float v = *value;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
   
 
  if(!SDDS_SetParameters(&(sddsFdTable[*fd].table),setMode,name->s,v,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setParameterValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}

long
  setParameterValueL(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long  *value		=(long *)argv[2];
   char *errMsg		=(char *)argv[3];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long  v = *value;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
   
 
  if(!SDDS_SetParameters(&(sddsFdTable[*fd].table),setMode,name->s,v,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setParameterValueD:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setParameterValueS(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   short  *value	=(short *)argv[2];
   char *errMsg		=(char *)argv[3];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   short  v = *value;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
   
  
  if(!SDDS_SetParameters(&(sddsFdTable[*fd].table),setMode,name->s,v,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setParameterValueS:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}

long
  setParameterValueB(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   char  *value		=(char *)argv[2];
   char *errMsg		=(char *)argv[3];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   char  v = *value;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
   
 
  if(!SDDS_SetParameters(&(sddsFdTable[*fd].table),setMode,name->s,v,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setParameterValueB:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}
long
  setParameterValueString(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   STRING  *value	=(STRING *)argv[2];
   char *errMsg		=(char *)argv[3];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   char *v = value->s;
   long setMode =(SDDS_SET_BY_NAME | SDDS_PASS_BY_VALUE);
   
 
  if(!SDDS_SetParameters(&(sddsFdTable[*fd].table),setMode,name->s,v,NULL))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("setParameterValueString:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
}

    /*****************************************************************
    result=getParameterValueX,fd,name,value,status,errmsg
		 result == long
		 fd == long (index into table)
		 name == string   (parametername)
		 value == (Double,Float,Long,Short,String,Byte)
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  getParameterValueD(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   double *value	=(double *)argv[2];
   short *status	=(short *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   double v;
  
  SDDS_SetColumnFlags(&(sddsFdTable[*fd].table),1);
  SDDS_SetRowFlags(&(sddsFdTable[*fd].table),1);
 
  if(SDDS_GetParameter(&(sddsFdTable[*fd].table),name->s,&v)== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterValueD:error  <%s>\n",errMsg);
	   return(0);
     }
  *value=v;
  *status=OK;
  return(0);
}

 long
  getParameterValueF(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   float  *value	=(float  *)argv[2];
   short *status	=(short *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   float v;
  
  if(SDDS_GetParameter(&(sddsFdTable[*fd].table),name->s,&v)== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterValueF:error  <%s>\n",errMsg);
	   return(0);
     }
  *value=v;
  *status=OK;
  return(0);
  }

 long
  getParameterValueL(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long  *value		=(long  *)argv[2];
   short *status	=(short *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long v;
  
  if(SDDS_GetParameter(&(sddsFdTable[*fd].table),name->s,&v)== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterValueL:error  <%s>\n",errMsg);
	   return(0);
     }
  *value=v;
  *status=OK;
  return(0);
  }

long
  getParameterValueS(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   short  *value	=(short  *)argv[2];
   short *status	=(short *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   short v;
  
  if(SDDS_GetParameter(&(sddsFdTable[*fd].table),name->s,&v)== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterValueS:error  <%s>\n",errMsg);
	   return(0);
     }
  *value=v;
  *status=OK;
  return(0);
  }
long
  getParameterValueB(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   char  *value		=(char  *)argv[2];
   short *status	=(short *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   char v;
  
  if(SDDS_GetParameter(&(sddsFdTable[*fd].table),name->s,&v)== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterValueB:error  <%s>\n",errMsg);
	   return(0);
     }
  *value=v;
  *status=OK;
  return(0);
  }

char *
  getParameterValueString(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   short *status	=(short *)argv[2];
   char *errMsg		=(char *)argv[3];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   char **v;
   char *retStr;
   long length;
   

   v=(char **)(SDDS_GetParameter(&(sddsFdTable[*fd].table),name->s,NULL));
   
  if(v == NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterValueString:error  <%s>\n",errMsg);
	   return(0);
     }
  length=strlen(*v);
  if( (retStr=(char*)malloc(length+1))==(char*)NULL)
	 {
	   strncpy(errMsg,"getParameterValue:malloc error",118);
	   errMsg[119]=0;
	   *status=ERROR;
	   if(DEBUG) printf("getParameterDefinition: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
  strncpy(retStr,*v,length);
  retStr[length]='\0';
  free(*v);
  *status=OK;
  return(retStr);
 }

/*****************************************************************
    result=setParameterDefinition,fd,name,symbol,units,description,
    format_string,fixed_value,type,errmsg
		 result == OK or ERROR 
		 fd == long (index into table)
		 name == string   (parametername)
		 symbol == string   (parametername)
		 units == string   (parametername)
		 description == string   (parametername)
		 format_string == string   (parametername)
		 fixed_value == string   (parametername)
		 type == long
		 errmsg == bytarr(120)
 ****************************************************************/
long
  setParameterDefinition(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   STRING *symbol	=(STRING *)argv[2];
   STRING *units	=(STRING *)argv[3];
   STRING *description	=(STRING *)argv[4];
    STRING *format_string	=(STRING *)argv[5];
   STRING *fixed_value	=(STRING *)argv[6];
   long *type		=(long *)argv[7];
   char *errMsg		=(char *)argv[8];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;

   if( SDDS_DefineParameter( &(sddsFdTable[*fd].table),
			     name->s,symbol->s,units->s,description->s,
			     format_string->s,*type,fixed_value->s ) == -1)
     {
       SDDS_PrintErrors(stdout,SDDS_VERBOSE_PrintErrors);
       sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
       strncpy(errMsg,sddsErrs[0],118);
       errMsg[119]=0;
       SDDS_ClearErrors();
       free(sddsErrs[0]);
       if(DEBUG) printf("defineParameter:error  <%s>\n",errMsg);
       return(ERROR);
     }
  return(OK);
 }
/*****************************************************************
    result=setColumnDefinition,fd,name,symbol,units,description,
    format_string,fixed_value,type,errmsg
		 result == OK or ERROR 
		 fd == long (index into table)
		 name == string   (columnname)
		 symbol == string   (column symbol)
		 units == string   (column units)
		 description == string   (column description)
		 format_string == string   (column format string)
		 field_length == long  (column field length)
		 type == long
		 errmsg == bytarr(120)
 ****************************************************************/
long
  setColumnDefinition(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   STRING *symbol	=(STRING *)argv[2];
   STRING *units	=(STRING *)argv[3];
   STRING *description	=(STRING *)argv[4];
   STRING *format_string	=(STRING *)argv[5];
   long *field_length	=(long *)argv[6];
   long *type		=(long *)argv[7];
   char *errMsg		=(char *)argv[8];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;

   if( SDDS_DefineColumn( &(sddsFdTable[*fd].table),
			     name->s,symbol->s,units->s,description->s,
			     format_string->s,*type,*field_length )==-1)
     {
       sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
       strncpy(errMsg,sddsErrs[0],118);
       errMsg[119]=0;
       SDDS_ClearErrors();
       free(sddsErrs[0]);
       if(DEBUG) printf("defineColumn:error  <%s>\n",errMsg);
       return(ERROR);
     }
   return(OK);
 }
/*****************************************************************
    result=setColumnDefinition,fd,name,symbol,units,description,
    format_string,fixed_value,type,errmsg
		 result == OK or ERROR 
		 fd == long (index into table)
		 name == string   (columnname)
		 symbol == string   (column symbol)
		 units == string   (column units)
		 description == string   (column description)
		 format_string == string   (column format string)
		 field_length == long  (column field length)
		 type == long
		 errmsg == bytarr(120)
 ****************************************************************/
long
  setArrayDefinition(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   STRING *symbol	=(STRING *)argv[2];
   STRING *units	=(STRING *)argv[3];
   STRING *description	=(STRING *)argv[4];
   STRING *format_string	=(STRING *)argv[5];
   long *field_length	=(long *)argv[6];
   long *dimensions	=(long *)argv[7];
   STRING *group_name	=(STRING *)argv[8];
   long *type		=(long *)argv[9];
   char *errMsg		=(char *)argv[10];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;

   if( SDDS_DefineArray( &(sddsFdTable[*fd].table),
			     name->s,symbol->s,units->s,description->s,
			     format_string->s,*type,*field_length,*dimensions,group_name->s )== -1)
     {
       sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
       strncpy(errMsg,sddsErrs[0],118);
       errMsg[119]=0;
       SDDS_ClearErrors();
       free(sddsErrs[0]);
       if(DEBUG) printf("defineArray:error  <%s>\n",errMsg);
       return(ERROR);
     }
   return(OK);
 }
	   
 /*****************************************************************
    result=getParameterDefinition,fd,name,type,status,errmsg
		 result == string 
		 fd == long (index into table)
		 name == string   (parametername)
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
char *
  getParameterDefinition(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long *type		=(long *)argv[2];
   short *status	=(short *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   PARAMETER_DEFINITION *pdef;
   char **sddsErrs;
   int32_t sddsNumErrs;
   int retlen;
   char *retStr;
   
 
   
  if( (pdef=SDDS_GetParameterDefinition(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
           if (sddsErrs == NULL) {
             sprintf(errMsg, "getParameterDefinition:error %s not found", name->s);
             *status=ERROR;
             return((char*)NULL);
           }
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterDefinition:error  <%s>\n",errMsg);
	   return((char*)NULL);
     }
   retlen=0;
    if(pdef->name!=NULL)retlen+=strlen(pdef->name);
    if(pdef->symbol!=NULL)retlen+=strlen(pdef->symbol);
   if(pdef->units!=NULL)retlen+=strlen(pdef->units);
   if(pdef->description!=NULL)retlen+=strlen(pdef->description);
   if(pdef->format_string!=NULL)retlen+=strlen(pdef->format_string);
   if(pdef->fixed_value!=NULL)retlen+=strlen(pdef->fixed_value);
				      
  if( (retStr=(char*)malloc(retlen+10))==(char*)NULL)
	 {
	   strncpy(errMsg,"getParameterDefinition:malloc error",118);
	   errMsg[119]=0;
	   *status=ERROR;
	   if(DEBUG) printf("getParameterDefinition: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
  retStr[0]=0;
  if(pdef->name!=NULL)strcat(retStr,pdef->name);
  strcat(retStr,"*");
   if(pdef->symbol!=NULL)strcat(retStr,pdef->symbol);
   strcat(retStr,"*");
   if(pdef->units!=NULL)strcat(retStr,pdef->units);
   strcat(retStr,"*");
   if(pdef->description!=NULL)strcat(retStr,pdef->description);
   strcat(retStr,"*");
   if(pdef->format_string!=NULL)strcat(retStr,pdef->format_string);
   strcat(retStr,"*");
   if(pdef->fixed_value!=NULL)strcat(retStr,pdef->fixed_value);
   *type=pdef->type;
   *status=OK;
   return(retStr);
  }
   /*****************************************************************
    result=getParameterType,fd,name
		 result == string 
		 fd == long (index into table)
		 name == string   (parametername)
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  getParameterType(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   return (SDDS_GetParameterInformation(&(sddsFdTable[*fd].table),"type",NULL,SDDS_GET_BY_NAME,name->s));
  }
   /*****************************************************************
    result=writeLayout,fd,
		 result == Ok or ERROR 
		 fd == long (index into table)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  writeLayout(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   char *errMsg		=(char *)argv[2];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
 
   

  if(!SDDS_WriteLayout(&(sddsFdTable[*fd].table)))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("write layout:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
  
  }
  /*****************************************************************
    result=startTable,fd,numberRows,errmsg
		 result == Ok or ERROR 
		 fd == long (index into table)
		 numberRows == long 
		 errmsg == bytarr(120)
 ****************************************************************/
long
  startTable(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   long  *numberRows	=(long  *)argv[1];
  char *errMsg		=(char *)argv[2];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
 
   

  if(!SDDS_StartTable(&(sddsFdTable[*fd].table),*numberRows))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("start table:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
  
  }

  /*****************************************************************
    result=writeTable,fd,errmsg
		 result == Ok or ERROR 
		 fd == long (index into table)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  writeTable(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
  char *errMsg		=(char *)argv[1];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
 
   

  if(!SDDS_WriteTable(&(sddsFdTable[*fd].table)))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("write table:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  return(OK);
  
  }
  /*****************************************************************
    result=terminate,fd,errmsg
		 result == Ok or ERROR 
		 fd == long (index into table)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  terminate(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
  char *errMsg		=(char *)argv[1];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
 
   

  if(!SDDS_Terminate(&(sddsFdTable[*fd].table)))
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("terminate:error  <%s>\n",errMsg);
	   return(ERROR);
     }
  sddsFdTable[*fd].free=1;
  sddsFdTable[*fd].currentPage=0;
  sddsFdTable[*fd].filename[0]='\0';
  sddsFdTable[*fd].mode=SDDS_READ;
  sddsFdTable[*fd].pageStarted=0;
  sddsFdTable[*fd].sddsArray.elements=0;
  if(sddsFdTable[*fd].sddsArray.data != NULL)      free(sddsFdTable[*fd].sddsArray.data);
  if(sddsFdTable[*fd].sddsArray.pointer != NULL)   free(sddsFdTable[*fd].sddsArray.pointer);
  if(sddsFdTable[*fd].sddsArray.dimension != NULL) free(sddsFdTable[*fd].sddsArray.dimension);
  sddsFdTable[*fd].sddsArray.data = NULL;
  sddsFdTable[*fd].sddsArray.pointer=NULL;
  sddsFdTable[*fd].sddsArray.dimension=NULL;
  
 return(OK);
  
  }



   /*****************************************************************
    result=getArrayDefinition,fd,name,type,field_length,dimensions,status,errmsg
		 result == string 
		 fd == long (index into table)
		 name == string   (parametername)
		 field_length = long
		 dimensions= long
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
char *
  getArrayDefinition(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long *type		=(long *)argv[2];
    long *field_length		=(long *)argv[3];
   long *dimensions		=(long *)argv[4];
  short *status	=(short *)argv[5];
   char *errMsg		=(char *)argv[6];

   /*
     Declare local variables
   */
   ARRAY_DEFINITION *pdef;
   char **sddsErrs;
   int32_t sddsNumErrs;
   int retlen;
   char *retStr;
   
 
   

  if( (pdef=SDDS_GetArrayDefinition(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
           if (sddsErrs == NULL) {
             sprintf(errMsg, "getArrayDefinition:error %s not found", name->s);
             *status=ERROR;
             return((char*)NULL);
           }
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getArrayDefinition:error  <%s>\n",errMsg);
	   return((char*)NULL);
     }
   retlen=0;
    if(pdef->name!=NULL)retlen+=strlen(pdef->name);
    if(pdef->symbol!=NULL)retlen+=strlen(pdef->symbol);
   if(pdef->units!=NULL)retlen+=strlen(pdef->units);
   if(pdef->description!=NULL)retlen+=strlen(pdef->description);
   if(pdef->format_string!=NULL)retlen+=strlen(pdef->format_string);
				      
  if( (retStr=(char*)malloc(retlen+10))==(char*)NULL)
	 {
	   strncpy(errMsg,"getArrayDefinition:malloc error",118);
	   errMsg[119]=0;
	   *status=ERROR;
	   if(DEBUG) printf("getArrayDefinition: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
  retStr[0]=0;
  if(pdef->name!=NULL)strcat(retStr,pdef->name);
  strcat(retStr,"*");
   if(pdef->symbol!=NULL)strcat(retStr,pdef->symbol);
   strcat(retStr,"*");
   if(pdef->units!=NULL)strcat(retStr,pdef->units);
   strcat(retStr,"*");
   if(pdef->description!=NULL)strcat(retStr,pdef->description);
   strcat(retStr,"*");
   if(pdef->format_string!=NULL)strcat(retStr,pdef->format_string);
   *type=pdef->type;
   *field_length=pdef->field_length;
   *dimensions=pdef->dimensions;
   *status=OK;
   return(retStr);
  }
  
   /*****************************************************************
    result=getArrayData,fd,name,arrayTypearrayDimensions,arrayDim,status,errmsg
		 result == #elements 
		 fd == long (index into table)
		 name == string   (parametername)
		 arrayType = long
		 arrayDimensions= long
		 arrayDim= long array[4]
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  getArrayData(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long *arrayType	=(long *)argv[2];
   long *arrayDimensions=(long *)argv[3];
   long *arrayDim	=(long *)argv[4];
   short *status	=(short *)argv[5];
   char *errMsg		=(char *)argv[6];
   
   /*
     Local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long arrayElements;
   int i;
   if(sddsFdTable[*fd].sddsArray.data != NULL)      free(sddsFdTable[*fd].sddsArray.data);
   if(sddsFdTable[*fd].sddsArray.pointer != NULL)   free(sddsFdTable[*fd].sddsArray.pointer);
   if(sddsFdTable[*fd].sddsArray.dimension != NULL) free(sddsFdTable[*fd].sddsArray.dimension);
   sddsFdTable[*fd].sddsArray.dimension=NULL;
   sddsFdTable[*fd].sddsArray.elements=0;
   sddsFdTable[*fd].sddsArray.data=NULL;
   sddsFdTable[*fd].sddsArray.pointer=NULL;
 
   if (SDDS_GetArray(&(sddsFdTable[*fd].table) ,name->s,&(sddsFdTable[*fd].sddsArray)) 
	== 0)
	 {
	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getArrayData: <%s>\n",errMsg);
	   return(ERROR);
	 }
  *arrayType=sddsFdTable[*fd].sddsArray.definition->type;
   *arrayDimensions=sddsFdTable[*fd].sddsArray.definition->dimensions;
   arrayElements=sddsFdTable[*fd].sddsArray.elements;
   for(i=0;i<*arrayDimensions;i++) arrayDim[i]=sddsFdTable[*fd].sddsArray.dimension[i];
   *status=OK;
   
   return(arrayElements);
 }

   
    /*****************************************************************
    result=getArrayValueX,fd,value,status,errmsg
		 result == long
		 fd == long (index into table)
		 value == (Double,Float,Long,Short,String,Byte)
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  getArrayValueD(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   double *value	=(double *)argv[1];
   short *status	=(short *)argv[2];

   /* Local variables */
   double *v;
   long i,elements;
   

   v=(double *)sddsFdTable[*fd].sddsArray.data;
   elements=sddsFdTable[*fd].sddsArray.elements;
   for(i=0;i<elements;i++) value[i]=v[i];
   free(sddsFdTable[*fd].sddsArray.data);
   free(sddsFdTable[*fd].sddsArray.pointer);
   free(sddsFdTable[*fd].sddsArray.dimension);
   sddsFdTable[*fd].sddsArray.dimension=NULL;
   sddsFdTable[*fd].sddsArray.elements=0;
   sddsFdTable[*fd].sddsArray.data=NULL;
   sddsFdTable[*fd].sddsArray.pointer=NULL;
   *status=OK;
  
  return(elements);
 }

long
  getArrayValueF(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   float *value		=(float *)argv[1];
   short *status	=(short *)argv[2];

   /* Local variables */
   float  *v;
   long i,elements;
   

   v=(float *)sddsFdTable[*fd].sddsArray.data;
   elements=sddsFdTable[*fd].sddsArray.elements;
   for(i=0;i<elements;i++) value[i]=v[i];
   free(sddsFdTable[*fd].sddsArray.data);
   free(sddsFdTable[*fd].sddsArray.pointer);
   free(sddsFdTable[*fd].sddsArray.dimension);
   sddsFdTable[*fd].sddsArray.dimension=NULL;
   sddsFdTable[*fd].sddsArray.elements=0;
   sddsFdTable[*fd].sddsArray.data=NULL;
   sddsFdTable[*fd].sddsArray.pointer=NULL;
   *status=OK;
  
  return(elements);
 }

long
  getArrayValueL(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   long  *value		=(long *)argv[1];
   short *status	=(short *)argv[2];

   /* Local variables */
   long  *v;
   long i,elements;
   

   v=(long  *)sddsFdTable[*fd].sddsArray.data;
   elements=sddsFdTable[*fd].sddsArray.elements;
   for(i=0;i<elements;i++) value[i]=v[i];
   free(sddsFdTable[*fd].sddsArray.data);
   free(sddsFdTable[*fd].sddsArray.pointer);
   free(sddsFdTable[*fd].sddsArray.dimension);
   sddsFdTable[*fd].sddsArray.dimension=NULL;
   sddsFdTable[*fd].sddsArray.elements=0;
   sddsFdTable[*fd].sddsArray.data=NULL;
   sddsFdTable[*fd].sddsArray.pointer=NULL;
   *status=OK;
  
  return(elements);
 }
long
  getArrayValueS(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   short  *value	=(short *)argv[1];
   short *status	=(short *)argv[2];

   /* Local variables */
   short  *v;
   long i,elements;
   

   v=(short  *)sddsFdTable[*fd].sddsArray.data;
   elements=sddsFdTable[*fd].sddsArray.elements;
   for(i=0;i<elements;i++) value[i]=v[i];
   free(sddsFdTable[*fd].sddsArray.data);
   free(sddsFdTable[*fd].sddsArray.pointer);
   free(sddsFdTable[*fd].sddsArray.dimension);
   sddsFdTable[*fd].sddsArray.dimension=NULL;
   sddsFdTable[*fd].sddsArray.elements=0;
   sddsFdTable[*fd].sddsArray.data=NULL;
   sddsFdTable[*fd].sddsArray.pointer=NULL;
   *status=OK;
  
  return(elements);
 }
long
  getArrayValueB(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   char  *value		=(char *)argv[1];
   short *status	=(short *)argv[2];

   /* Local variables */
   char  *v;
   long i,elements;
   

   v=(char  *)sddsFdTable[*fd].sddsArray.data;
   elements=sddsFdTable[*fd].sddsArray.elements;
   for(i=0;i<elements;i++) value[i]=v[i];
   free(sddsFdTable[*fd].sddsArray.data);
   free(sddsFdTable[*fd].sddsArray.pointer);
   free(sddsFdTable[*fd].sddsArray.dimension);
   sddsFdTable[*fd].sddsArray.dimension=NULL;
   sddsFdTable[*fd].sddsArray.elements=0;
   sddsFdTable[*fd].sddsArray.data=NULL;
   sddsFdTable[*fd].sddsArray.pointer=NULL;
   *status=OK;
  
   return(elements);
 }

char *
getArrayValueString(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   short *status	=(short *)argv[1];
   char *errMsg		=(char *)argv[2];

   /*
     Declare local variables
   */
   char **v;
   long length,i,elements;
   char *retStr;


   v=(char  **)sddsFdTable[*fd].sddsArray.data;
   elements=sddsFdTable[*fd].sddsArray.elements;
   length=0;
   for(i=0;i<elements;i++)
     {
       length+=strlen(v[i]);
     }
  if( (retStr=(char*)malloc(length + elements + 2))==NULL)
	 {
	   strncpy(errMsg,"getarrayvalue:mallc error",118);
	   errMsg[119]=0;
	   *status=ERROR;
	   if(DEBUG) printf("getArrayvalueString: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
  retStr[0]=0;
  for(i=0;i<elements;i++) {
    strcat(retStr,v[i]);
    if(i != (elements-1)) strcat(retStr,"*");
    free(v[i]);
  }
   free(sddsFdTable[*fd].sddsArray.data);
   free(sddsFdTable[*fd].sddsArray.pointer);
   free(sddsFdTable[*fd].sddsArray.dimension);
   sddsFdTable[*fd].sddsArray.dimension=NULL;
   sddsFdTable[*fd].sddsArray.elements=0;
   sddsFdTable[*fd].sddsArray.data=NULL;
   sddsFdTable[*fd].sddsArray.pointer=NULL;
  *status=OK;
   return(retStr);
   
  }
 
       
    /*****************************************************************
    result=getColumnRows,fd,status,errmsg
		 result == #rows 
		 fd == long (index into table)
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  getColumnRows(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   short *status	=(short *)argv[1];
   char *errMsg		=(char *)argv[2];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   long nRows;
 
   

  SDDS_SetColumnFlags(&(sddsFdTable[*fd].table),1);
  SDDS_SetRowFlags(&(sddsFdTable[*fd].table),1);
  
  if( (nRows=SDDS_CountRowsOfInterest(&(sddsFdTable[*fd].table)))==-1) 
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getColumnRows:error  <%s>\n",errMsg);
     }
  return(nRows);
 }

  /*****************************************************************
    result=getColumnDefinition,fd,name,type,field_length,status,errmsg
		 result == string 
		 fd == long (index into table)
		 name == string   (parametername)
		 field_length = long
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
char *
  getColumnDefinition(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long *type		=(long *)argv[2];
   long *field_length	=(long *)argv[3];
   short *status	=(short *)argv[4];
   char *errMsg		=(char *)argv[5];

   /*
     Declare local variables
   */
   COLUMN_DEFINITION *pdef;
   char **sddsErrs;
   int32_t sddsNumErrs;
   int retlen;
   char *retStr;
   
 
   

  if( (pdef=SDDS_GetColumnDefinition(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
           if (sddsErrs == NULL) {
             sprintf(errMsg, "getColumnDefinition:error %s not found", name->s);
             *status=ERROR;
             return((char*)NULL);
           }
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getColumnDefinition:error  <%s>\n",errMsg);
	   return((char*)NULL);
     }
   retlen=0;
   if(pdef->name!=NULL)retlen+=strlen(pdef->name);
   if(pdef->symbol!=NULL)retlen+=strlen(pdef->symbol);
   if(pdef->units!=NULL)retlen+=strlen(pdef->units);
   if(pdef->description!=NULL)retlen+=strlen(pdef->description);
   if(pdef->format_string!=NULL)retlen+=strlen(pdef->format_string);
				      
  if( (retStr=(char*)malloc(retlen+10))==(char*)NULL)
	 {
	   strncpy(errMsg,"getColumnDefinition:malloc error",118);
	   errMsg[119]=0;
	   *status=ERROR;
	   if(DEBUG) printf("getColumnDefinition: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
  retStr[0]=0;
  if(pdef->name!=NULL)strcat(retStr,pdef->name);
  strcat(retStr,"*");
   if(pdef->symbol!=NULL)strcat(retStr,pdef->symbol);
   strcat(retStr,"*");
   if(pdef->units!=NULL)strcat(retStr,pdef->units);
   strcat(retStr,"*");
   if(pdef->description!=NULL)strcat(retStr,pdef->description);
   strcat(retStr,"*");
   if(pdef->format_string!=NULL)strcat(retStr,pdef->format_string);
   *type=pdef->type;
   *field_length=pdef->field_length;
   *status=OK;
   return(retStr);
  }

    /*****************************************************************
    result=getColumnValueX,fd,name,value,status,errmsg
		 result == long
		 fd == long (index into table)
		 name == string   (parametername)
		 value == (Double,Float,Long,Short,String,Byte)
		 status == short   (ERROR or OK)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  getColumnValueD(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   double *value	=(double *)argv[2];
   long  *nrows		=(long *)argv[3];
   short *status	=(short *)argv[4];
   char *errMsg		=(char *)argv[5];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   double *v;
   int i;
  
  SDDS_SetColumnFlags(&(sddsFdTable[*fd].table),1);
  SDDS_SetRowFlags(&(sddsFdTable[*fd].table),1);
 
  if( (v=SDDS_GetColumn(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterValueD:error  <%s>\n",errMsg);
	   return(0);
     }
  for(i=0;i<*nrows;i++)
    {
      value[i]=v[i];
    }
   *status=OK;
   free(v);
  return(0);
}

 long
  getColumnValueF(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   float  *value	=(float  *)argv[2];
   long  *nrows		=(long  *)argv[3];
   short *status	=(short *)argv[4];
   char *errMsg		=(char *)argv[5];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   float *v;
   int i;
  
  if( (v=SDDS_GetColumn(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getColumnValueF:error  <%s>\n",errMsg);
	   return(0);
     }
  for(i=0;i<*nrows;i++)
    {
      value[i]=v[i];
    }
   free(v);
  *status=OK;
  return(0);
  }

 long
  getColumnValueL(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long  *value		=(long  *)argv[2];
   long  *nrows		=(long  *)argv[3];
   short *status	=(short *)argv[4];
   char *errMsg		=(char *)argv[5];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   int32_t *v;
   int i;
  
  if( (v=SDDS_GetColumn(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getParameterValueL:error  <%s>\n",errMsg);
	   return(0);
     }
  for(i=0;i<*nrows;i++)
    {
      value[i]=(int32_t)v[i];
    }
   free(v);
  *status=OK;
  return(0);
  }

long
  getColumnValueS(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   short  *value	=(short  *)argv[2];
   long  *nrows		=(long  *)argv[3];
   short *status	=(short *)argv[4];
   char *errMsg		=(char *)argv[5];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   short *v;
    int i;
 
  if( (v=SDDS_GetColumn(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getColumnValueS:error  <%s>\n",errMsg);
	   return(0);
     }
  for(i=0;i<*nrows;i++)
    {
      value[i]=v[i];
    }
   free(v);
  *status=OK;
  return(0);
  }

long
  getColumnValueB(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   char  *value		=(char  *)argv[2];
   long  *nrows		=(long  *)argv[3];
   short *status	=(short *)argv[4];
   char *errMsg		=(char *)argv[5];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   char  *v;
   int i;
 
  if( (v=SDDS_GetColumn(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getColumnValueS:error  <%s>\n",errMsg);
	   return(0);
     }
  for(i=0;i<*nrows;i++)
    {
      value[i]=v[i];
    }
   free(v);
  *status=OK;
  return(0);
  }

char *
  getColumnValueString(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   STRING *name		=(STRING *)argv[1];
   long  *nrows		=(long  *)argv[2];
   short *status	=(short *)argv[3];
   char *errMsg		=(char *)argv[4];

   /*
     Declare local variables
   */
   char **sddsErrs;
   int32_t sddsNumErrs;
   char **v;
   long length;
   char *retStr;
    int i;
  

  
  if( (v=SDDS_GetColumn(&(sddsFdTable[*fd].table),name->s))== NULL)
     {
 	   sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	   strncpy(errMsg,sddsErrs[0],118);
	   errMsg[119]=0;
	   *status=ERROR;
	   SDDS_ClearErrors();
	   free(sddsErrs[0]);
	   if(DEBUG) printf("getColumnValueString:error  <%s>\n",errMsg);
	   return(0);
     }
  length=0;
  for(i=0;i<*nrows;i++)
    {
      length+=strlen(v[i]);
    }
  if( (retStr=(char*)malloc(length + (*nrows) + 2))==NULL)
	 {
	   strncpy(errMsg,"getcolumvalue:mallc error",118);
	   errMsg[119]=0;
	   *status=ERROR;
	   if(DEBUG) printf("getColumnrvalueString: <%s>\n",errMsg);
	   return((char*)NULL);
	 }
  retStr[0]=0;
  for(i=0;i<*nrows-1;i++) {
    strcat(retStr,v[i]);
    strcat(retStr,"*");
    free(v[i]);
  }
  strcat(retStr,v[*nrows-1]);
  free(v[*nrows-1]);
   free(v);
  *status=OK;
  return(retStr);
  }
  /*****************************************************************
    result=setPage,fd,requestedPage,status,errmsg
		 result == long (current page)
		 fd == long (index into table)
		 requestedPage== short
		 status == short   (ERROR or OK of EOF)
		 errmsg == bytarr(120)
 ****************************************************************/
long
  setPage(argc, argv)
  int argc;
 void *argv[];
 {
   /*
     Declare passed in values from IDL call
   */
   long *fd		=(long *)argv[0];
   short *requestedPage	=(short *)argv[1];
   short *status	=(short *)argv[2];
   char *errMsg		=(char *)argv[3];

   /*
     Declare local variables
   */
   long		pagesToGo;
   long tableRead;
   char **sddsErrs;
   int32_t sddsNumErrs;
   int i;
  
   

  if(*requestedPage == -1) *requestedPage=sddsFdTable[*fd].currentPage+1;
  if(*requestedPage < sddsFdTable[*fd].currentPage) {
    if(!SDDS_Terminate(&(sddsFdTable[*fd].table)))
      {
	sddsFdTable[*fd].free=1;
	sddsFdTable[*fd].currentPage=0;
	*status=ERROR;
	sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	strcpy(errMsg, sddsErrs[0]);
	SDDS_ClearErrors();
	free(sddsErrs[0]);
	if(DEBUG) printf("setPage error<%s>\n",errMsg);
	return(0);
      }
    if (!SDDS_InitializeInput(&(sddsFdTable[*fd].table), sddsFdTable[*fd].filename))
      {
	sddsFdTable[*fd].free=1;
	sddsFdTable[*fd].currentPage=0;
	sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	strcpy(errMsg, sddsErrs[0]);
	SDDS_ClearErrors();
	free(sddsErrs[0]);
	*status=ERROR;
	if(DEBUG) printf("setPage  error<%s>\n",errMsg);
	return(0);
      }
    sddsFdTable[*fd].currentPage=0;
  }
  pagesToGo=*requestedPage-sddsFdTable[*fd].currentPage;
  for(i=0;i<pagesToGo;i++)
    {
      if( (tableRead=SDDS_ReadTable(&(sddsFdTable[*fd].table))) <=0)
	{
	  if(tableRead == -1)
	    {
	      *status=OK;
	      return(-1);
	    }
	  else
	    {
	      sddsErrs = SDDS_GetErrorMessages(&sddsNumErrs, SDDS_ERROR_MODE);
	      strcpy(errMsg, sddsErrs[0]);
	      SDDS_ClearErrors();
	      free(sddsErrs[0]);
	      *status=ERROR;
	      if(DEBUG) printf("setPage  error<%s>\n",errMsg);
	      return(sddsFdTable[*fd].currentPage);
	    }
	}
      sddsFdTable[*fd].currentPage++;
    }
  *status=OK;
  return( sddsFdTable[*fd].currentPage);
 }
